/*----------------------------------------------------------------------- 

File Name: VectorSequenceFactories.c

<lalVerbatim file="VectorSequenceFactoriesCV">
Revision: $Id: VectorSequenceFactories.m4,v 1.2 2000/06/03 21:36:37 jolien Exp $
</lalVerbatim>

-------------------------------------------------------------------------*/

/* <lalLaTeX>

\subsection{Module \texttt{VectorSequenceFactories.c}}
\label{ss:VectorSequenceFactories.c}

Create/destroy $\langle\mbox{datatype}\rangle$VectorSequence objects. 

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{VectorSequenceFactoriesD}
\index{\verb&ZCreateVectorSequence()&}
\index{\verb&CCreateVectorSequence()&}
\index{\verb&DCreateVectorSequence()&}
\index{\verb&SCreateVectorSequence()&}
\index{\verb&I2CreateVectorSequence()&}
\index{\verb&I4CreateVectorSequence()&}
\index{\verb&I8CreateVectorSequence()&}
\index{\verb&U2CreateVectorSequence()&}
\index{\verb&U4CreateVectorSequence()&}
\index{\verb&U8CreateVectorSequence()&}
\index{\verb&CHARCreateVectorSequence()&}
\index{\verb&CreateVectorSequence()&}
\index{\verb&ZDestroyVectorSequence()&}
\index{\verb&CDestroyVectorSequence()&}
\index{\verb&DDestroyVectorSequence()&}
\index{\verb&SDestroyVectorSequence()&}
\index{\verb&I2DestroyVectorSequence()&}
\index{\verb&I4DestroyVectorSequence()&}
\index{\verb&I8DestroyVectorSequence()&}
\index{\verb&U2DestroyVectorSequence()&}
\index{\verb&U4DestroyVectorSequence()&}
\index{\verb&U8DestroyVectorSequence()&}
\index{\verb&CHARDestroyVectorSequence()&}
\index{\verb&DestroyVectorSequence()&}

\subsubsection*{Description}

The \texttt{CreateVectorSequence} family of functions create a
$\langle\mbox{datatype}\rangle$\texttt{VectorSequence} of the
appropriate dimensions.

The \texttt{DestroyVectorSequence} family of functions return the storage
allocated by the \texttt{CreateVectorSequence} functions to the system.

\subsubsection*{Algorithm}

\subsubsection*{Uses}
\begin{verbatim}
LALMalloc()
LALFree()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{VectorSequenceFactoriesCV}}

</lalLaTeX> */

#include "LALStdlib.h"
#include "SeqFactories.h"

/* <lalVerbatim file="VectorSequenceFactoriesNRCSID"> */
NRCSID( VECTORSEQUENCEFACTORIESC, "$Id: VectorSequenceFactories.m4,v 1.2 2000/06/03 21:36:37 jolien Exp $" );
/* </lalVerbatim> */

















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void ZCreateVectorSequence ( Status *status, COMPLEX16VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "ZCreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( COMPLEX16VectorSequence * ) LALMalloc( sizeof( COMPLEX16VectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( COMPLEX16 );
    (*vseq)->data = ( COMPLEX16 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void ZDestroyVectorSequence ( Status *status, COMPLEX16VectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "ZDestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void CCreateVectorSequence ( Status *status, COMPLEX8VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "CCreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( COMPLEX8VectorSequence * ) LALMalloc( sizeof( COMPLEX8VectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( COMPLEX8 );
    (*vseq)->data = ( COMPLEX8 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void CDestroyVectorSequence ( Status *status, COMPLEX8VectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "CDestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void DCreateVectorSequence ( Status *status, REAL8VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "DCreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( REAL8VectorSequence * ) LALMalloc( sizeof( REAL8VectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( REAL8 );
    (*vseq)->data = ( REAL8 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void DDestroyVectorSequence ( Status *status, REAL8VectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "DDestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void SCreateVectorSequence ( Status *status, REAL4VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "SCreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( REAL4VectorSequence * ) LALMalloc( sizeof( REAL4VectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( REAL4 );
    (*vseq)->data = ( REAL4 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void SDestroyVectorSequence ( Status *status, REAL4VectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "SDestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void I2CreateVectorSequence ( Status *status, INT2VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "I2CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( INT2VectorSequence * ) LALMalloc( sizeof( INT2VectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( INT2 );
    (*vseq)->data = ( INT2 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void I2DestroyVectorSequence ( Status *status, INT2VectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "I2DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void I4CreateVectorSequence ( Status *status, INT4VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "I4CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( INT4VectorSequence * ) LALMalloc( sizeof( INT4VectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( INT4 );
    (*vseq)->data = ( INT4 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void I4DestroyVectorSequence ( Status *status, INT4VectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "I4DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void I8CreateVectorSequence ( Status *status, INT8VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "I8CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( INT8VectorSequence * ) LALMalloc( sizeof( INT8VectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( INT8 );
    (*vseq)->data = ( INT8 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void I8DestroyVectorSequence ( Status *status, INT8VectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "I8DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void U2CreateVectorSequence ( Status *status, UINT2VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "U2CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( UINT2VectorSequence * ) LALMalloc( sizeof( UINT2VectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( UINT2 );
    (*vseq)->data = ( UINT2 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void U2DestroyVectorSequence ( Status *status, UINT2VectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "U2DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void U4CreateVectorSequence ( Status *status, UINT4VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "U4CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( UINT4VectorSequence * ) LALMalloc( sizeof( UINT4VectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( UINT4 );
    (*vseq)->data = ( UINT4 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void U4DestroyVectorSequence ( Status *status, UINT4VectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "U4DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void U8CreateVectorSequence ( Status *status, UINT8VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "U8CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( UINT8VectorSequence * ) LALMalloc( sizeof( UINT8VectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( UINT8 );
    (*vseq)->data = ( UINT8 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void U8DestroyVectorSequence ( Status *status, UINT8VectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "U8DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void CHARCreateVectorSequence ( Status *status, CHARVectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "CHARCreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( CHARVectorSequence * ) LALMalloc( sizeof( CHARVectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( CHAR );
    (*vseq)->data = ( CHAR * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void CHARDestroyVectorSequence ( Status *status, CHARVectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "CHARDestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void CreateVectorSequence ( Status *status, REAL4VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */
  INITSTATUS( status, "CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, SEQFACTORIESH_EINPTR, SEQFACTORIESH_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          SEQFACTORIESH_ESLENGTH, SEQFACTORIESH_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          SEQFACTORIESH_EVLENGTH, SEQFACTORIESH_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR);
  ASSERT (*vseq == NULL, status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( REAL4VectorSequence * ) LALMalloc( sizeof( REAL4VectorSequence ) );
  ASSERT (*vseq != NULL, status,
          SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( REAL4 );
    (*vseq)->data = ( REAL4 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, SEQFACTORIESH_EMALLOC, SEQFACTORIESH_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















/* <lalVerbatim file="VectorSequenceFactoriesD"> */
void DestroyVectorSequence ( Status *status, REAL4VectorSequence **vseq )
{ /* </lalVerbatim> */
  /* 
   * Initialize status
   */

  INITSTATUS( status, "DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, SEQFACTORIESH_EVPTR, SEQFACTORIESH_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, SEQFACTORIESH_EUPTR, SEQFACTORIESH_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          SEQFACTORIESH_EDPTR, SEQFACTORIESH_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}

